#include "mpsoa.h"


struct swarm pop;

/* default options */
struct Options opt = { 40,      /* swarm size */
		       25,      /* progress */
		       2.0,     /* cognitial parameter */
		       2.0,     /* social parameter */
		       0.5,     /* maximum velocity factor */
		       2000,    /* maximum of iterations */
		       0.9,     /* initial weight */
		       0.4,     /* final weight */
		       0.5,     /* max norm 2 for gradient */
		       0,       /* use a halton initial population */
		       1,       /* multilocal optimization */
		       2        /* full info */
};

int primes[]={2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 53, 59,
	      61, 67, 71, 73, 79, 83, 89, 97, 101, 103, 107, 109, 113, 127,
	      131, 137, 139, 149, 151, 157, 163, 167, 173, 179, 181, 191, 193,
	      197, 199, 211, 223, 227, 229, 233, 239, 241, 251, 257, 263, 269,
	      271, 277, 281, 283, 293, 307, 311, 313, 317, 331, 337, 347, 349,
	      353, 359, 367, 373, 379, 383, 389, 397, 401, 409, 419, 421, 431,
	      433, 439, 443, 449, 457, 461, 463, 467, 479, 487, 491};

int nprimes=sizeof(primes)/sizeof(int);



int mlocpsoa(int n, double (*objf)(), double *lb, double *ub, double **sol,
	     double *f)
{
  int i,j, gbest, mode;
  int progress, iter; /* to measure progress */
  double objvalue, *objgrd, *maxv;
  double weight;
  char *buff;

  time_t tt;


  time(&tt);
  buff=ctime(&tt);
  if(opt.verb)printf("\nInitial time: %s\n",buff);

  /* seed random number with time */
  srand((unsigned int)tt);



  /* allocate memory for swarm */
  pop.x    =malloc(opt.s*n*sizeof(double));
  pop.v    =malloc(opt.s*n*sizeof(double));
  pop.y    =malloc(opt.s*n*sizeof(double));
  pop.fy   =malloc(opt.s * sizeof(double));

  if(opt.mlocal)
    pop.ygrd =malloc(opt.s*n*sizeof(double));
  else
    pop.ygrd =NULL;

  if(opt.mlocal){    /* allocate memory for gradient */
    objgrd=malloc(n*sizeof(double));
    mode=2;      /* requests objective and gradient */
  } else {
    objgrd=NULL;
    mode=0;      /* requests only the objective function value */
  }

  /* allocate memory for maximum velocity allowed */
  maxv=malloc(n*sizeof(double));

  if(!pop.x || !pop.v || !pop.y || (opt.mlocal && !pop.ygrd) || !pop.fy ||
     (opt.mlocal && !objgrd) || ! maxv)
    return EXIT_MEM;


  /* initialize population */
  if(opt.halton){ /* pseudo-random halton population */
    int split, prime;

    split=(int)floor(nprimes/n);

    for(j=0;j<n;j++){
      prime=primes[j*split+(int)((rand()/(RAND_MAX+1.0))*(split-1))];
      for(i=0;i<opt.s;i++)
	pop.x[i*n+j]=lb[j]+halton(prime, i+1)*(ub[j]-lb[j]);
    }

    for(i=0;i<opt.s;i++)
      pop.fy[i]=-Inf;  /* in first iteration, y and ygrd will be set */

  } else { /* uniform pseudo-random population */
    
    for(i=0;i<opt.s;i++){ /* for all particle   */
      for(j=0;j<n;j++){   /* for all dimensions */
	pop.x[i*n+j]=(rand()/(RAND_MAX+1.0))*(ub[j]-lb[j])+lb[j];
      }
      pop.fy[i]=-Inf; /* in first iteration, y and ygrd will be set */
    }
  }



  /* initialize  velocity and maximum velocity allowed */
  memset(pop.v, 0, opt.s*n*sizeof(double));
  for(j=0;j<n;j++)
    maxv[j]=(ub[j]-lb[j])*opt.maxvfactor;


  progress=0; /* with progress */
  iter=0;
  gbest=0; /* global best */

  /* Main cycle */
  while(progress<opt.prog && iter < opt.maxiter){

    progress++;
    iter++;

    /* inertia factor is a linear interpolation from iweight to fweight */
    weight = (opt.iweight-opt.fweight)*
      ((double)(opt.maxiter-iter))/((double)opt.maxiter)+opt.fweight;

    for(i=0;i<opt.s;i++){ /* for each particle */

      /* compute objective and gradient */
      objvalue=objf(n, &pop.x[i*n], mode, objgrd);

      if(pop.fy[i]<objvalue){ /* progress obtained */
	progress=0; /* reset noprogress counter */
	pop.fy[i]=objvalue;
	memcpy(&pop.y[i*n], &pop.x[i*n], n*sizeof(double));
	if(mode)
	  memcpy(&pop.ygrd[i*n], objgrd, n*sizeof(double));
	if(pop.fy[gbest]<pop.fy[i])
	  gbest=i; /* global best indice */
      }

      /* update velocity */
      for(j=0;j<n;j++){
	pop.v[i*n+j] = weight*pop.v[i*n+j] +
	  opt.mu*(rand()/(RAND_MAX+1.0))*(pop.y[i*n+j]-pop.x[i*n+j]);

	if(mode)
	  pop.v[i*n+j] += opt.nu*(rand()/(RAND_MAX+1.0))*(objgrd[j]);
	else
	  pop.v[i*n+j] +=
	    opt.nu*(rand()/(RAND_MAX+1.0))*(pop.y[gbest*n+j]-pop.x[i*n+j]);

	if (pop.v[i*n+j]>maxv[j])
	  pop.v[i*n+j]=maxv[j];
	else if (pop.v[i*n+j]<-maxv[j])
	  pop.v[i*n+j]=-maxv[j];
      }

      /* update particle and check bound limits */
      for(j=0;j<n;j++){
	pop.x[i*n+j]+=pop.v[i*n+j];
	if(pop.x[i*n+j]>ub[j])
	  pop.x[i*n+j]=ub[j];
	else if(pop.x[i*n+j]<lb[j])
	  pop.x[i*n+j]=lb[j];
      }
      
    }

  }

  if(opt.verb)printf("%d iterations\n", iter);

  time(&tt);
  buff=ctime(&tt);
  if(opt.verb)printf("Final time: %s\n", buff);

  /* returning the best of the population */
  *sol=malloc(n*sizeof(double));
  if(!(*sol))
    return EXIT_MEM;

  memcpy(*sol,&pop.y[gbest*n],n*sizeof(double));

  if(opt.verb>1)print_pop(n, opt.s, &pop);

  /* free allocated memory */
  free(pop.x);
  free(pop.y);
  free(pop.v);
  free(pop.fy);
  free(maxv);
  if(pop.ygrd)
    free(pop.ygrd);
  if(objgrd)
    free(objgrd);

  return EXIT_OK;
}




/* print the best of each particle in a population */
void print_pop(int n, int s, struct swarm *pop)
{
  int i, j;

  printf("Printing the best so far for each particle\n");

  for(i=0;i<s;i++){ /* for each particle */
    printf("p(%d)=[", i+1);
    for(j=0;j<n-1;j++) /* for each coordinate */
      printf("%.6f,", pop->y[i*n+j]);
    printf("%.6f];\n", pop->y[i*n+n-1]);
    printf("f(%d)=%.6f\n", i+1, pop->fy[i]);
  }
  printf("\n");

}




double halton(int p, int k)
{
  double hval;
  int pot, j;
  div_t division;
  
  hval=0;
  pot=1;
  for(j=1;j<=15;j++){
    division=div((int)floor(k/pot), p);
    pot*=p;    
    hval+=(double)(division.rem)/pot;
  }
  return hval;
}
