###############################################################################
# Objective: Linear
# Constraints: Simple bounds
###############################################################################
# Pollution control problem
#    Coded by A. Ismael F. Vaz 31/03/2004  aivaz@dps.uminho.pt
#    University of Minho, Portugal
#
# Policy abatement problem described by S.-A. Gustafson and K.O. Kortanek in 
#  "Analytical properties of some multiple-source urban diffusion models",
#  Environment and Planning 4, 1972, pp. 31-41.
#
# Adapted for Gaussion model without plume rise.
#
# Adapted for multilocal optimization by A. Ismael F. Vaz 14/04/2004
#
# This is a regular (finite) maximization problem ...
#
###############################################################################

# Pi
param pi := 4*atan(1);
#number of pollution emission sources
param nsources;
# Wind direction
param theta;
# Mean wind speed
param U;
# Position of sources
param xpos{1..nsources};
param ypos{1..nsources};
# emission rates
param Q{1..nsources};
# Stack height
param h{1..nsources};




#number of t
param nt;

# Pollution abatement
param r{1..nsources};

# t variable t[1]-> x coordenate
# t variable t[2]-> y coordenate
var t {i in 1..nt};

# Translation and rotation for sigma and concentration
var X{i in 1..nsources} = (t[1]-xpos[i])*cos(theta)-(t[2]-ypos[i])*sin(theta);
var Y{i in 1..nsources} = (t[1]-xpos[i])*sin(theta)+(t[2]-ypos[i])*cos(theta);

# Standard deviations
var sigmaY{i in 1..nsources} = if(X[i]>0) then (X[i]/2)^0.5 else 0;

var sigmaZ{i in 1..nsources} = if(X[i]>0) then (X[i]/2)^0.5 else 0;

var sigmaZY{i in 1..nsources} = if(X[i]>0) then (X[i]/2) else 0;

# Concentration of gas
var C {i in 1..nsources} = if(sigmaY[i]!=0 && sigmaZ[i]!=0)then
			     (Q[i]/(pi*sigmaZY[i]*U))*
			     exp(-0.5*((Y[i]/sigmaY[i])^2+(h[i]/sigmaZ[i])^2))
			   else 0;



maximize ft:
	sum {i in 1..nsources} ((1-r[i])*C[i]);

subject to tbound1:
	-1 <= t[1] <= 4;
subject to tbound2:
	-1 <= t[2] <= 4;

data;

param nsources=3;
param nt := 2;

param theta := 0;                     # Wind direction in rad
param U     := .02533029591058444286; # Mean wind speed  (1/(2*pi))^2


param xpos :=		# x position on the ground (m)
	1	 0
	2	 0
	3	 2;

param ypos :=		# y position on the ground (m)
	1	 1
	2	 0
	3	 -1;


param h :=		# Effective stack height
	1	 1
	2	 1
	3	 1.41421356237309504880;  # sqrt(2);


param Q :=		# Uniform emission rate
	1	1
	2	1
	3	1;


param r := 
	1 0.987495
	2 0.951230
	3 0.942638;

option solver mlocpsoa;
solve;

##################
# Solution found #
##################
printf "Solution found\n";
display t;
display ft;
